import pytest
import json
import os
import time
from logic import QuestionnaireProcessor


def test_init_empty() -> None:
    '''Test whether after initializing without arguments the questionnaire variable is set correctly.'''
    empty_init_questionnairePro = QuestionnaireProcessor()
    with open("../questionnaire_templates/Processing_Template_Neuro.json") as file:
        correct_empty_questionnare = json.load(file)
    assert correct_empty_questionnare == empty_init_questionnairePro.questionnaire

def test_init_with_path() -> None:
    '''Test whether after initializing with a path the questionnaire variable is set correctly.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    with open("../questionnaire_templates/test_questionnaire") as file:
        correct_test_questionnare = json.load(file)
    assert correct_test_questionnare == questionnairePro.questionnaire

def test_load_unprocessed_empty_arg() -> None: 
    '''Test whether the first file in the ../unprocessed directory is loaded by the 
    loadUnprocessedPatientdata method if no arguments are given.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    with open("../unprocessed_files/" + os.listdir("../unprocessed_files/")[0]) as file:
        unprocessed = json.load(file)
    assert unprocessed == questionnairePro.loadUnprocessedPatientdata()

def test_load_unprocessed_path1() -> None: 
    '''Test whether for a given path loadUnprocessedPatientdata returns the dictionary in said path.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    with open("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87c0.json") as file:
        unprocessed = json.load(file)
    assert unprocessed == questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87c0.json")

def test_load_unprocessed_path2() -> None: 
    '''Test whether for a given path loadUnprocessedPatientdata returns the dictionary in said path.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    with open("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87ae.json") as file:
        unprocessed = json.load(file)
    assert unprocessed == questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87ae.json")

def test_processed_headder_includes_correct_keys() -> None:
    '''Test that all neccessary keys are included and have propper types'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    keys = list(processed.keys())
    for key in {"first_name", "last_name", "questionnaireresponseid"}:
        key in keys
        assert isinstance(processed[key],str)

def test_processed_headder_has_correct_values() -> None:
    '''Test for the correct values'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    assert processed["questionnaireresponseid"] == questionnairePro.questionnaire["id"]
    assert processed["first_name"] == ""
    assert processed["last_name"] == unprocessed["contained"][0]["name"][0]["family"]

def test_processed_includes_nothing_additional() -> None:
    '''Test no keys are improperly included in the processed data and the dictionary is not empty'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    assert processed
    for key in processed:
        assert key in {"questionnaireresponseid", "first_name", "last_name", "prios", "Patient_Priority", "entrance"}

def test_prios_key_exists() -> None:
    '''Test that the prios key is in the processed data'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    assert "prios" in processed

def test_prios_includes_no_additional_keys() -> None:
    '''Test that prios includes no keys beyond the ones it should have and that it is not empty.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    prios = processed["prios"]
    assert prios # should not be empty
    for question in prios:
        for key in question:
            assert key in {"linkId", "id", "prio", "text", "answer"}
    
def test_prios_have_correct_structure() -> None:
    '''Test that the prios have the intended structure of holding a string under linkId, id, text, prio 
    and a list under answer that contains dictionaries with the key valueString'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    prios = processed["prios"]
    for item in prios:
        for key in {"linkId", "id", "prio", "text"}:
            assert key in item
            assert isinstance(item[key],str)
        assert "answer" in item
        assert isinstance(item["answer"],list)
        assert item["answer"] # not empty
        for answer in item["answer"]:
            assert isinstance(answer,dict)
            assert list(answer) == ["valueString"]
            assert isinstance(answer["valueString"],str)

def test_prios_have_correct_values() -> None:
    ''' Test the values are copied over correctly and given the correct prios even if multiple answers were given. '''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    prios = processed["prios"]
    assert prios[0]["linkId"] == "question neutral"
    assert prios[0]["id"] == "e6f93635-cbd6-468b-9fb6-1fabde7b879a"
    assert prios[0]["text"] == "question neutral"
    assert prios[0]["prio"] == "yellow"
    assert prios[0]["answer"][0]["valueString"] == "Answer1"
    assert prios[4]["answer"][1]["valueString"] == "Answer3"

def test_priority_key_exists() -> None:
    '''Test that the Patient Priority key exists and has a string Value. '''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    assert "Patient_Priority" in processed
    assert isinstance(processed["Patient_Priority"],str)

def test_patient_priority_correct() -> None:
    '''Test for unprocessed files that the patient priority was calculated corectly.'''
    correct_prios = ["yellow", "red", "green", "red", "red", "red", "red", "red", "red", "green", "green", "red", "red", "red", "red", "red", "red", "green", "red", "green"]
    for idx, fileending in enumerate(list(map(hex,range(int("ae",16),int("c2",16))))):
        questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
        unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87"+fileending[-2:]+".json")
        processed = questionnairePro.processPatientdata(unprocessed)
        assert processed["Patient_Priority"] == correct_prios[idx]

def test_entrance_exists() -> None:
    '''Test that the entrance time key exists and has a float type value. '''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    assert "entrance" in processed
    assert isinstance(processed["entrance"],float)

def test_entrance_time_correct_value() -> None:
    '''Test that the entrance time of a patient that was just processed is nearly the current time.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    assert int(time.time()) == int(processed["entrance"])


def test_save_processed_data() -> None:
    '''Test that if processed data is saved the file exists and has the correct data in it.'''
    questionnairePro = QuestionnaireProcessor("../questionnaire_templates/test_questionnaire")
    unprocessed = questionnairePro.loadUnprocessedPatientdata("../unprocessed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    processed = questionnairePro.processPatientdata(unprocessed)
    questionnairePro.saveProcessedData(processed, "../processed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    assert os.path.exists("../processed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")
    with open("../processed_files/e6f93635-cbd6-468b-9fb6-1fabde7b87b0.json")as file:
        assert json.load(file) == processed


